@@ -7,6 +7,7 @@
  *
  * @package WordPress
  * @subpackage Feed
+ * @since 2.1.0
  */
 
 /**
@@ -22,10 +23,10 @@
  * @param string $show See get_bloginfo() for possible values.
  * @return string
  */
-function get_bloginfo_rss($show = '') {
-	$info = strip_tags(get_bloginfo($show));
+function get_bloginfo_rss( $show = '' ) {
+	$info = strip_tags( get_bloginfo( $show ) );
 	/**
-	 * Filter the bloginfo for use in RSS feeds.
+	 * Filters the bloginfo for use in RSS feeds.
 	 *
 	 * @since 2.2.0
 	 *
@@ -50,9 +51,9 @@ function get_bloginfo_rss($show = '') {
  *
  * @param string $show See get_bloginfo() for possible values.
  */
-function bloginfo_rss($show = '') {
+function bloginfo_rss( $show = '' ) {
 	/**
-	 * Filter the bloginfo for display in RSS feeds.
+	 * Filters the bloginfo for display in RSS feeds.
 	 *
 	 * @since 2.1.0
 	 *
@@ -68,7 +69,7 @@ function bloginfo_rss($show = '') {
  * Retrieve the default feed.
  *
  * The default feed is 'rss2', unless a plugin changes it through the
- * 'default_feed' filter.
+ * {@see 'default_feed'} filter.
  *
  * @since 2.5.0
  *
@@ -76,7 +77,7 @@ function bloginfo_rss($show = '') {
  */
 function get_default_feed() {
 	/**
-	 * Filter the default feed type.
+	 * Filters the default feed type.
 	 *
 	 * @since 2.5.0
 	 *
@@ -98,12 +99,12 @@ function get_default_feed() {
  */
 function get_wp_title_rss( $deprecated = '&#8211;' ) {
 	if ( '&#8211;' !== $deprecated ) {
-		/* translators: %s: 'document_title_separator' filter name */
+		/* translators: %s: 'document_title_separator' filter name. */
 		_deprecated_argument( __FUNCTION__, '4.4.0', sprintf( __( 'Use the %s filter instead.' ), '<code>document_title_separator</code>' ) );
 	}
 
 	/**
-	 * Filter the blog title for use as the feed title.
+	 * Filters the blog title for use as the feed title.
 	 *
 	 * @since 2.2.0
 	 * @since 4.4.0 The `$sep` parameter was deprecated and renamed to `$deprecated`.
@@ -124,12 +125,12 @@ function get_wp_title_rss( $deprecated = '&#8211;' ) {
  */
 function wp_title_rss( $deprecated = '&#8211;' ) {
 	if ( '&#8211;' !== $deprecated ) {
-		/* translators: %s: 'document_title_separator' filter name */
+		/* translators: %s: 'document_title_separator' filter name. */
 		_deprecated_argument( __FUNCTION__, '4.4.0', sprintf( __( 'Use the %s filter instead.' ), '<code>document_title_separator</code>' ) );
 	}
 
 	/**
-	 * Filter the blog title for display of the feed title.
+	 * Filters the blog title for display of the feed title.
 	 *
 	 * @since 2.2.0
 	 * @since 4.4.0 The `$sep` parameter was deprecated and renamed to `$deprecated`.
@@ -153,7 +154,7 @@ function get_the_title_rss() {
 	$title = get_the_title();
 
 	/**
-	 * Filter the post title for use in a feed.
+	 * Filters the post title for use in a feed.
 	 *
 	 * @since 1.2.0
 	 *
@@ -181,15 +182,16 @@ function the_title_rss() {
  * @param string $feed_type The type of feed. rss2 | atom | rss | rdf
  * @return string The filtered content.
  */
-function get_the_content_feed($feed_type = null) {
-	if ( !$feed_type )
+function get_the_content_feed( $feed_type = null ) {
+	if ( ! $feed_type ) {
 		$feed_type = get_default_feed();
+	}
 
 	/** This filter is documented in wp-includes/post-template.php */
 	$content = apply_filters( 'the_content', get_the_content() );
-	$content = str_replace(']]>', ']]&gt;', $content);
+	$content = str_replace( ']]>', ']]&gt;', $content );
 	/**
-	 * Filter the post content for use in feeds.
+	 * Filters the post content for use in feeds.
 	 *
 	 * @since 2.9.0
 	 *
@@ -207,8 +209,8 @@ function get_the_content_feed($feed_type = null) {
  *
  * @param string $feed_type The type of feed. rss2 | atom | rss | rdf
  */
-function the_content_feed($feed_type = null) {
-	echo get_the_content_feed($feed_type);
+function the_content_feed( $feed_type = null ) {
+	echo get_the_content_feed( $feed_type );
 }
 
 /**
@@ -219,7 +221,7 @@ function the_content_feed($feed_type = null) {
 function the_excerpt_rss() {
 	$output = get_the_excerpt();
 	/**
-	 * Filter the post excerpt for a feed.
+	 * Filters the post excerpt for a feed.
 	 *
 	 * @since 1.2.0
 	 *
@@ -235,7 +237,7 @@ function the_excerpt_rss() {
  */
 function the_permalink_rss() {
 	/**
-	 * Filter the permalink to the post for use in feeds.
+	 * Filters the permalink to the post for use in feeds.
 	 *
 	 * @since 2.3.0
 	 *
@@ -252,7 +254,7 @@ function the_permalink_rss() {
  */
 function comments_link_feed() {
 	/**
-	 * Filter the comments permalink for the current post.
+	 * Filters the comments permalink for the current post.
 	 *
 	 * @since 3.6.0
 	 *
@@ -269,8 +271,8 @@ function comments_link_feed() {
  *
  * @param int|WP_Comment $comment_id Optional comment object or id. Defaults to global comment object.
  */
-function comment_guid($comment_id = null) {
-	echo esc_url( get_comment_guid($comment_id) );
+function comment_guid( $comment_id = null ) {
+	echo esc_url( get_comment_guid( $comment_id ) );
 }
 
 /**
@@ -281,13 +283,14 @@ function comment_guid($comment_id = null) {
  * @param int|WP_Comment $comment_id Optional comment object or id. Defaults to global comment object.
  * @return false|string false on failure or guid for comment on success.
  */
-function get_comment_guid($comment_id = null) {
-	$comment = get_comment($comment_id);
+function get_comment_guid( $comment_id = null ) {
+	$comment = get_comment( $comment_id );
 
-	if ( !is_object($comment) )
+	if ( ! is_object( $comment ) ) {
 		return false;
+	}
 
-	return get_the_guid($comment->comment_post_ID) . '#comment-' . $comment->comment_ID;
+	return get_the_guid( $comment->comment_post_ID ) . '#comment-' . $comment->comment_ID;
 }
 
 /**
@@ -300,7 +303,7 @@ function get_comment_guid($comment_id = null) {
  */
 function comment_link( $comment = null ) {
 	/**
-	 * Filter the current comment's permalink.
+	 * Filters the current comment's permalink.
 	 *
 	 * @since 3.6.0
 	 *
@@ -320,7 +323,7 @@ function comment_link( $comment = null ) {
  */
 function get_comment_author_rss() {
 	/**
-	 * Filter the current comment author for use in a feed.
+	 * Filters the current comment author for use in a feed.
 	 *
 	 * @since 1.5.0
 	 *
@@ -348,7 +351,7 @@ function comment_author_rss() {
 function comment_text_rss() {
 	$comment_text = get_comment_text();
 	/**
-	 * Filter the current comment content for use in a feed.
+	 * Filters the current comment content for use in a feed.
 	 *
 	 * @since 1.5.0
 	 *
@@ -370,39 +373,46 @@ function comment_text_rss() {
  * @param string $type Optional, default is the type returned by get_default_feed().
  * @return string All of the post categories for displaying in the feed.
  */
-function get_the_category_rss($type = null) {
-	if ( empty($type) )
+function get_the_category_rss( $type = null ) {
+	if ( empty( $type ) ) {
 		$type = get_default_feed();
+	}
 	$categories = get_the_category();
-	$tags = get_the_tags();
-	$the_list = '';
-	$cat_names = array();
+	$tags       = get_the_tags();
+	$the_list   = '';
+	$cat_names  = array();
 
 	$filter = 'rss';
-	if ( 'atom' == $type )
+	if ( 'atom' == $type ) {
 		$filter = 'raw';
-
-	if ( !empty($categories) ) foreach ( (array) $categories as $category ) {
-		$cat_names[] = sanitize_term_field('name', $category->name, $category->term_id, 'category', $filter);
 	}
 
-	if ( !empty($tags) ) foreach ( (array) $tags as $tag ) {
-		$cat_names[] = sanitize_term_field('name', $tag->name, $tag->term_id, 'post_tag', $filter);
+	if ( ! empty( $categories ) ) {
+		foreach ( (array) $categories as $category ) {
+			$cat_names[] = sanitize_term_field( 'name', $category->name, $category->term_id, 'category', $filter );
+		}
 	}
 
-	$cat_names = array_unique($cat_names);
+	if ( ! empty( $tags ) ) {
+		foreach ( (array) $tags as $tag ) {
+			$cat_names[] = sanitize_term_field( 'name', $tag->name, $tag->term_id, 'post_tag', $filter );
+		}
+	}
+
+	$cat_names = array_unique( $cat_names );
 
 	foreach ( $cat_names as $cat_name ) {
-		if ( 'rdf' == $type )
+		if ( 'rdf' == $type ) {
 			$the_list .= "\t\t<dc:subject><![CDATA[$cat_name]]></dc:subject>\n";
-		elseif ( 'atom' == $type )
+		} elseif ( 'atom' == $type ) {
 			$the_list .= sprintf( '<category scheme="%1$s" term="%2$s" />', esc_attr( get_bloginfo_rss( 'url' ) ), esc_attr( $cat_name ) );
-		else
-			$the_list .= "\t\t<category><![CDATA[" . @html_entity_decode( $cat_name, ENT_COMPAT, get_option('blog_charset') ) . "]]></category>\n";
+		} else {
+			$the_list .= "\t\t<category><![CDATA[" . html_entity_decode( $cat_name, ENT_COMPAT, get_option( 'blog_charset' ) ) . "]]></category>\n";
+		}
 	}
 
 	/**
-	 * Filter all of the post categories for display in a feed.
+	 * Filters all of the post categories for display in a feed.
 	 *
 	 * @since 1.2.0
 	 *
@@ -421,8 +431,8 @@ function get_the_category_rss($type = null) {
  *
  * @param string $type Optional, default is the type returned by get_default_feed().
  */
-function the_category_rss($type = null) {
-	echo get_the_category_rss($type);
+function the_category_rss( $type = null ) {
+	echo get_the_category_rss( $type );
 }
 
 /**
@@ -433,11 +443,12 @@ function the_category_rss($type = null) {
  * @since 2.2.0
  */
 function html_type_rss() {
-	$type = get_bloginfo('html_type');
-	if (strpos($type, 'xhtml') !== false)
+	$type = get_bloginfo( 'html_type' );
+	if ( strpos( $type, 'xhtml' ) !== false ) {
 		$type = 'xhtml';
-	else
+	} else {
 		$type = 'html';
+	}
 	echo $type;
 }
 
@@ -456,26 +467,27 @@ function html_type_rss() {
  * @since 1.5.0
  */
 function rss_enclosure() {
-	if ( post_password_required() )
+	if ( post_password_required() ) {
 		return;
+	}
 
-	foreach ( (array) get_post_custom() as $key => $val) {
-		if ($key == 'enclosure') {
+	foreach ( (array) get_post_custom() as $key => $val ) {
+		if ( $key == 'enclosure' ) {
 			foreach ( (array) $val as $enc ) {
-				$enclosure = explode("\n", $enc);
+				$enclosure = explode( "\n", $enc );
 
 				// only get the first element, e.g. audio/mpeg from 'audio/mpeg mpga mp2 mp3'
-				$t = preg_split('/[ \t]/', trim($enclosure[2]) );
+				$t    = preg_split( '/[ \t]/', trim( $enclosure[2] ) );
 				$type = $t[0];
 
 				/**
-				 * Filter the RSS enclosure HTML link tag for the current post.
+				 * Filters the RSS enclosure HTML link tag for the current post.
 				 *
 				 * @since 2.2.0
 				 *
 				 * @param string $html_link_tag The HTML link tag with a URI and other attributes.
 				 */
-				echo apply_filters( 'rss_enclosure', '<enclosure url="' . trim( htmlspecialchars( $enclosure[0] ) ) . '" length="' . trim( $enclosure[1] ) . '" type="' . $type . '" />' . "\n" );
+				echo apply_filters( 'rss_enclosure', '<enclosure url="' . esc_url( trim( $enclosure[0] ) ) . '" length="' . absint( trim( $enclosure[1] ) ) . '" type="' . esc_attr( $type ) . '" />' . "\n" );
 			}
 		}
 	}
@@ -495,21 +507,22 @@ function rss_enclosure() {
  * @since 2.2.0
  */
 function atom_enclosure() {
-	if ( post_password_required() )
+	if ( post_password_required() ) {
 		return;
+	}
 
 	foreach ( (array) get_post_custom() as $key => $val ) {
-		if ($key == 'enclosure') {
+		if ( $key == 'enclosure' ) {
 			foreach ( (array) $val as $enc ) {
-				$enclosure = explode("\n", $enc);
+				$enclosure = explode( "\n", $enc );
 				/**
-				 * Filter the atom enclosure HTML link tag for the current post.
+				 * Filters the atom enclosure HTML link tag for the current post.
 				 *
 				 * @since 2.2.0
 				 *
 				 * @param string $html_link_tag The HTML link tag with a URI and other attributes.
 				 */
-				echo apply_filters( 'atom_enclosure', '<link href="' . trim( htmlspecialchars( $enclosure[0] ) ) . '" rel="enclosure" length="' . trim( $enclosure[1] ) . '" type="' . trim( $enclosure[2] ) . '" />' . "\n" );
+				echo apply_filters( 'atom_enclosure', '<link href="' . esc_url( trim( $enclosure[0] ) ) . '" rel="enclosure" length="' . absint( trim( $enclosure[1] ) ) . '" type="' . esc_attr( trim( $enclosure[2] ) ) . '" />' . "\n" );
 			}
 		}
 	}
@@ -532,29 +545,35 @@ function atom_enclosure() {
  * @param string $data Input string
  * @return array array(type, value)
  */
-function prep_atom_text_construct($data) {
-	if (strpos($data, '<') === false && strpos($data, '&') === false) {
-		return array('text', $data);
+function prep_atom_text_construct( $data ) {
+	if ( strpos( $data, '<' ) === false && strpos( $data, '&' ) === false ) {
+		return array( 'text', $data );
+	}
+
+	if ( ! function_exists( 'xml_parser_create' ) ) {
+		trigger_error( __( "PHP's XML extension is not available. Please contact your hosting provider to enable PHP's XML extension." ) );
+
+		return array( 'html', "<![CDATA[$data]]>" );
 	}
 
 	$parser = xml_parser_create();
-	xml_parse($parser, '<div>' . $data . '</div>', true);
-	$code = xml_get_error_code($parser);
-	xml_parser_free($parser);
+	xml_parse( $parser, '<div>' . $data . '</div>', true );
+	$code = xml_get_error_code( $parser );
+	xml_parser_free( $parser );
 
-	if (!$code) {
-		if (strpos($data, '<') === false) {
-			return array('text', $data);
+	if ( ! $code ) {
+		if ( strpos( $data, '<' ) === false ) {
+			return array( 'text', $data );
 		} else {
 			$data = "<div xmlns='http://www.w3.org/1999/xhtml'>$data</div>";
-			return array('xhtml', $data);
+			return array( 'xhtml', $data );
 		}
 	}
 
-	if (strpos($data, ']]>') === false) {
-		return array('html', "<![CDATA[$data]]>");
+	if ( strpos( $data, ']]>' ) === false ) {
+		return array( 'html', "<![CDATA[$data]]>" );
 	} else {
-		return array('html', htmlspecialchars($data));
+		return array( 'html', htmlspecialchars( $data ) );
 	}
 }
 
@@ -568,7 +587,7 @@ function prep_atom_text_construct($data) {
 function atom_site_icon() {
 	$url = get_site_icon_url( 32 );
 	if ( $url ) {
-		echo "<icon>$url</icon>\n";
+		echo '<icon>' . convert_chars( $url ) . "</icon>\n";
 	}
 }
 
@@ -596,6 +615,18 @@ function rss2_site_icon() {
 	}
 }
 
+/**
+ * Returns the link for the currently displayed feed.
+ *
+ * @since 5.3.0
+ *
+ * @return string Correct link for the atom:self element.
+ */
+function get_self_link() {
+	$host = @parse_url( home_url() );
+	return set_url_scheme( 'http://' . $host['host'] . wp_unslash( $_SERVER['REQUEST_URI'] ) );
+}
+
 /**
  * Display the link for the currently displayed feed in a XSS safe way.
  *
@@ -604,9 +635,8 @@ function rss2_site_icon() {
  * @since 2.5.0
  */
 function self_link() {
-	$host = @parse_url(home_url());
 	/**
-	 * Filter the current feed URL.
+	 * Filters the current feed URL.
 	 *
 	 * @since 3.6.0
 	 *
@@ -615,7 +645,61 @@ function self_link() {
 	 *
 	 * @param string $feed_link The link for the feed with set URL scheme.
 	 */
-	echo esc_url( apply_filters( 'self_link', set_url_scheme( 'http://' . $host['host'] . wp_unslash( $_SERVER['REQUEST_URI'] ) ) ) );
+	echo esc_url( apply_filters( 'self_link', get_self_link() ) );
+}
+
+/**
+ * Get the timestamp of the most recently modified post from WP_Query.
+ *
+ * If viewing a comment feed, the timestamp of the most recently modified
+ * comment will be returned.
+ *
+ * @global WP_Query $wp_query WordPress Query object.
+ *
+ * @since 5.2.0
+ *
+ * @param string $format Format of the timestamp to return, passed to mysql2date.
+ *
+ * @return string The timestamp.
+ */
+function get_feed_build_date( $format ) {
+	global $wp_query;
+
+	if ( empty( $wp_query ) || ! $wp_query->have_posts() ) {
+		// Fallback to last time any post was modified or published.
+		return get_lastpostmodified( 'GMT' );
+	}
+
+	// Extract the post modified times from the posts.
+	$modified_times = wp_list_pluck( $wp_query->posts, 'post_modified_gmt' );
+
+	// If this is a comment feed, check those objects too.
+	if ( $wp_query->is_comment_feed() && $wp_query->comment_count ) {
+		// Extract the comment modified times from the comments.
+		$comment_times = wp_list_pluck( $wp_query->comments, 'comment_date_gmt' );
+
+		// Add the comment times to the post times for comparison.
+		$modified_times = array_merge( $modified_times, $comment_times );
+	}
+
+	// Determine the maximum modified time.
+	$datetime = date_create_immutable_from_format(
+		'Y-m-d H:i:s',
+		max( $modified_times ),
+		new DateTimeZone( 'UTC' )
+	);
+
+	$max_modified_time = $datetime->format( $format );
+
+	/**
+	 * Filters the date the last post or comment in the query was modified.
+	 *
+	 * @since 5.2.0
+	 *
+	 * @param string $max_modified_time Date the last post or comment was modified in the query.
+	 * @param string $format            The date format requested in get_feed_build_date.
+	 */
+	return apply_filters( 'get_feed_build_date', $max_modified_time, $format );
 }
 
 /**
@@ -626,21 +710,22 @@ function self_link() {
  * @param string $type Type of feed. Possible values include 'rss', rss2', 'atom', and 'rdf'.
  */
 function feed_content_type( $type = '' ) {
-	if ( empty($type) )
+	if ( empty( $type ) ) {
 		$type = get_default_feed();
+	}
 
 	$types = array(
 		'rss'      => 'application/rss+xml',
 		'rss2'     => 'application/rss+xml',
 		'rss-http' => 'text/xml',
 		'atom'     => 'application/atom+xml',
-		'rdf'      => 'application/rdf+xml'
+		'rdf'      => 'application/rdf+xml',
 	);
 
-	$content_type = ( !empty($types[$type]) ) ? $types[$type] : 'application/octet-stream';
+	$content_type = ( ! empty( $types[ $type ] ) ) ? $types[ $type ] : 'application/octet-stream';
 
 	/**
-	 * Filter the content type for a specific feed type.
+	 * Filters the content type for a specific feed type.
 	 *
 	 * @since 2.8.0
 	 *
@@ -657,12 +742,19 @@ function feed_content_type( $type = '' ) {
  *
  * @param mixed $url URL of feed to retrieve. If an array of URLs, the feeds are merged
  * using SimplePie's multifeed feature.
- * See also {@link ​http://simplepie.org/wiki/faq/typical_multifeed_gotchas}
+ * See also {@link http://simplepie.org/wiki/faq/typical_multifeed_gotchas}
  *
  * @return WP_Error|SimplePie WP_Error object on failure or SimplePie object on success
  */
 function fetch_feed( $url ) {
-	require_once( ABSPATH . WPINC . '/class-feed.php' );
+	if ( ! class_exists( 'SimplePie', false ) ) {
+		require_once( ABSPATH . WPINC . '/class-simplepie.php' );
+	}
+
+	require_once( ABSPATH . WPINC . '/class-wp-feed-cache.php' );
+	require_once( ABSPATH . WPINC . '/class-wp-feed-cache-transient.php' );
+	require_once( ABSPATH . WPINC . '/class-wp-simplepie-file.php' );
+	require_once( ABSPATH . WPINC . '/class-wp-simplepie-sanitize-kses.php' );
 
 	$feed = new SimplePie();
 
@@ -675,23 +767,23 @@ function fetch_feed( $url ) {
 	$feed->set_file_class( 'WP_SimplePie_File' );
 
 	$feed->set_feed_url( $url );
-	/** This filter is documented in wp-includes/class-feed.php */
+	/** This filter is documented in wp-includes/class-wp-feed-cache-transient.php */
 	$feed->set_cache_duration( apply_filters( 'wp_feed_cache_transient_lifetime', 12 * HOUR_IN_SECONDS, $url ) );
 	/**
 	 * Fires just before processing the SimplePie feed object.
 	 *
 	 * @since 3.0.0
 	 *
-	 * @param object &$feed SimplePie feed object, passed by reference.
-	 * @param mixed  $url   URL of feed to retrieve. If an array of URLs, the feeds are merged.
+	 * @param object $feed SimplePie feed object (passed by reference).
+	 * @param mixed  $url  URL of feed to retrieve. If an array of URLs, the feeds are merged.
 	 */
 	do_action_ref_array( 'wp_feed_options', array( &$feed, $url ) );
 	$feed->init();
 	$feed->set_output_encoding( get_option( 'blog_charset' ) );
-	$feed->handle_content_type();
 
-	if ( $feed->error() )
+	if ( $feed->error() ) {
 		return new WP_Error( 'simplepie-error', $feed->error() );
+	}
 
 	return $feed;
 }
